@@ -1,13 +1,12 @@
 import numpy
 
 from syned.beamline.element_coordinates import ElementCoordinates
-from syned.beamline.shape import Plane
 
+from shadow4.beam.s4_beam import S4Beam
 from shadow4.beamline.optical_elements.gratings.s4_grating import S4GratingElement, S4Grating
-from shadow4.beamline.s4_optical_element import S4PlaneOpticalElement
+from shadow4.beamline.s4_optical_element import S4PlaneOpticalElementDecorator
 
-
-class S4PlaneGrating(S4Grating, S4PlaneOpticalElement):
+class S4PlaneGrating(S4Grating, S4PlaneOpticalElementDecorator):
     def __init__(self,
                  name="Undefined",
                  boundary_shape=None,
@@ -28,7 +27,7 @@ class S4PlaneGrating(S4Grating, S4PlaneOpticalElement):
                  f_ruling=0,
                  ):
 
-        S4PlaneOpticalElement.__init__(self)
+        S4PlaneOpticalElementDecorator.__init__(self)
         S4Grating.__init__(self,
                            name=name,
                            surface_shape=self.get_surface_shape_instance(),
@@ -54,24 +53,25 @@ class S4PlaneGrating(S4Grating, S4PlaneOpticalElement):
     #     return S4Conic.initialize_as_plane()
 
 class S4PlaneGratingElement(S4GratingElement):
-    def __init__(self, optical_element=None, coordinates=None):
-        super().__init__(optical_element if optical_element is not None else S4Grating(),
-                         coordinates if coordinates is not None else ElementCoordinates())
-        if not isinstance(self.get_optical_element().get_surface_shape(), Plane):
-            raise ValueError("Wrong Optical Element: only Plane shape is accepted")
-
+    def __init__(self,
+                 optical_element : S4PlaneGrating = None,
+                 coordinates : ElementCoordinates = None,
+                 input_beam : S4Beam = None):
+        super().__init__(optical_element=optical_element if optical_element is not None else S4PlaneGrating(),
+                         coordinates=coordinates if coordinates is not None else ElementCoordinates(),
+                         input_beam=input_beam)
 
 if __name__ == "__main__":
 
     from shadow4.sources.source_geometrical.source_gaussian import SourceGaussian
-    from shadow4.beam.beam import Beam
+    from shadow4.beam.s4_beam import S4Beam
     from shadow4.tools.graphics import plotxy
 
     #
     # source
     #
     src = SourceGaussian.initialize_from_keywords(
-                 number_of_rays=10000,
+                 nrays=10000,
                  sigmaX=1.0e-6,
                  sigmaY=0.0,
                  sigmaZ=1.0e-6,
@@ -80,7 +80,7 @@ if __name__ == "__main__":
                  real_space_center=[0.0,0.0,0.0],
                  direction_space_center=[0.0,0.0]
                                  )
-    beam = Beam()
+    beam = S4Beam()
 
     beam.genSource(src)
     beam.set_photon_energy_eV(1000.0)
@@ -119,10 +119,10 @@ if __name__ == "__main__":
 
 
 
-    ge = S4PlaneGratingElement(optical_element=g, coordinates=coordinates_syned)
+    ge = S4PlaneGratingElement(optical_element=g, coordinates=coordinates_syned, input_beam=beam)
 
     print(ge.info())
 
-    beam_out = ge.trace_beam(beam)
+    beam_out = ge.trace_beam()
     plotxy(beam_out[0], 1, 3, title="Image 0", nbins=201)
 
