@@ -1,11 +1,13 @@
 import numpy
 
-from syned.beamline.shape import Sphere, SphericalCylinder, Convexity, Direction
+from syned.beamline.shape import Convexity, Direction
 
-from shadow4.beamline.s4_optical_element import SurfaceCalculation, S4SphereOpticalElement
+from shadow4.beam.s4_beam import S4Beam
+from shadow4.beamline.s4_optical_element import SurfaceCalculation, S4SphereOpticalElementDecorator
 from shadow4.beamline.optical_elements.gratings.s4_grating import S4GratingElement, S4Grating, ElementCoordinates
 
-class S4SphereGrating(S4Grating, S4SphereOpticalElement):
+
+class S4SphereGrating(S4Grating, S4SphereOpticalElementDecorator):
     def __init__(self,
                  name="Sphere Grating",
                  boundary_shape=None,
@@ -35,8 +37,8 @@ class S4SphereGrating(S4Grating, S4SphereOpticalElement):
                  grazing_angle=0.0,
                  ):
 
-        S4SphereOpticalElement.__init__(self, surface_calculation, is_cylinder, cylinder_direction, convexity,
-                                        radius, p_focus, q_focus, grazing_angle)
+        S4SphereOpticalElementDecorator.__init__(self, surface_calculation, is_cylinder, cylinder_direction, convexity,
+                                                 radius, p_focus, q_focus, grazing_angle)
         S4Grating.__init__(self,
                            name=name,
                            surface_shape=self.get_surface_shape_instance(),
@@ -59,12 +61,13 @@ class S4SphereGrating(S4Grating, S4SphereOpticalElement):
                            )
 
 class S4SphereGratingElement(S4GratingElement):
-    def __init__(self, optical_element=None, coordinates=None):
-        super().__init__(optical_element if optical_element is not None else S4SphereMirror(),
-                         coordinates if coordinates is not None else ElementCoordinates())
-        if not (isinstance(self.get_optical_element().get_surface_shape(), SphericalCylinder) or
-                isinstance(self.get_optical_element().get_surface_shape(), Sphere)):
-            raise ValueError("Wrong Optical Element: only Sphere or Spherical Cylinder shape is accepted")
+    def __init__(self,
+                 optical_element : S4SphereGrating = None,
+                 coordinates : ElementCoordinates = None,
+                 input_beam : S4Beam = None):
+        super().__init__(optical_element=optical_element if optical_element is not None else S4SphereGrating(),
+                         coordinates=coordinates if coordinates is not None else ElementCoordinates(),
+                         input_beam=input_beam)
 
     # def apply_grating_diffraction(self, beam):
     #     return self.get_optical_element().apply_grating_diffraction(beam)
@@ -80,6 +83,7 @@ if __name__ == "__main__":
     src = SourceGeometrical(spatial_type="Point",
                     angular_distribution = "Flat",
                     energy_distribution = "Uniform",
+                    nrays = 5000,
                             )
 
     src.set_angular_distribution_flat(0,0,0,0)
@@ -88,7 +92,7 @@ if __name__ == "__main__":
 
     # print(src.info())
 
-    beam = src.get_beam(N=5000, POL_DEG=1.0, POL_ANGLE=0.0, F_COHER=False)
+    beam = src.get_beam()
 
 
     print(beam.info())
@@ -133,10 +137,10 @@ if __name__ == "__main__":
 
 
 
-    ge = S4SphereGratingElement(optical_element=g, coordinates=coordinates_syned)
+    ge = S4SphereGratingElement(optical_element=g, coordinates=coordinates_syned, input_beam=beam)
 
     print(ge.info())
 
-    beam_out = ge.trace_beam(beam)
+    beam_out = ge.trace_beam()
 
     plotxy(beam_out[0], 1, 3, title="Image 0", nbins=201)
