@@ -1,13 +1,13 @@
 
 import numpy
-from shadow4.beam.beam import Beam
+from shadow4.beam.s4_beam import S4Beam
 from shadow4.sources.s4_light_source_base import S4LightSourceBase
 
 class SourceGridPolar(S4LightSourceBase):
 
     def __init__(self,
-                 real_space=[1e-6,0,1e-6],
-                 direction_space=[1e-6,1e-6],
+                 real_space_width=[1e-6,0,1e-6],
+                 direction_space_width=[1e-6,1e-6],
                  real_space_points=[100,36],
                  direction_space_points=[1,1],
                  real_space_center=[0,0,0],
@@ -15,13 +15,16 @@ class SourceGridPolar(S4LightSourceBase):
                  name="Undefined",
                  nrays=0, # not used
                  seed=0, # not used
+                 wavelength=1e-10,
+                 polarization_degree=1.0,
+                 polarization_phase_deg=0.0,
                  ):
         """
         This defines a grid source, so points starting in a ellipsoid-like volume in real space
         and angularly gridded in X,Z
 
-        :param real_space: the widths of the real_space volume [2a,2b,2c] of the ellipsoid
-        :param direction_space: The "angular" aperture [Dx',Dz']
+        :param real_space_width: the widths of the real_space_width volume [2a,2b,2c] of the ellipsoid
+        :param direction_space_width: The "angular" aperture [Dx',Dz']
         :param real_space_points: Number of points [Nradial,Nangular]
         :param direction_space_points: Number of points [Nradial',Nangular']
         :param real_space_center: Center in real space in cartesial coordinates [Xc,Yc,Zc]
@@ -31,22 +34,24 @@ class SourceGridPolar(S4LightSourceBase):
 
         super().__init__(name=name, nrays=nrays, seed=seed)
 
-        if real_space[1] != 0:
+        if real_space_width[1] != 0:
             raise Exception("Finite source depth not yet implemented!") # TODO: implement it!
 
-        self._real_space = real_space
-        self._direction_space = direction_space
+        self._real_space_width = real_space_width
+        self._direction_space_width = direction_space_width
         self._real_space_points = real_space_points
         self._direction_space_points = direction_space_points
         self._real_space_center = real_space_center
         self._direction_space_center = direction_space_center
-
+        self._wavelength = wavelength
+        self._polarization_degree = polarization_degree
+        self._polarization_phase_deg = polarization_phase_deg
 
 
     @classmethod
     def initialize_point_source(cls,
                 real_space_center=[0.0, 0.0, 0.0],
-                direction_space=[1e-6,1e-6],
+                direction_space_width=[1e-6,1e-6],
                 direction_space_points=[5,36],
                 direction_space_center=[0.0,0.0],
                                 ):
@@ -54,13 +59,13 @@ class SourceGridPolar(S4LightSourceBase):
         Initializes a point source
 
         :param real_space_center: Center in real space in cartesial coordinates [Xc,Yc,Zc]
-        :param direction_space: Default: [1e-6,1e-6],
+        :param direction_space_width: Default: [1e-6,1e-6],
         :param direction_space_points: Default: [100,100],
         :param direction_space_center: Default: [0.0,0.0] ),
         :return:
         """
-        return SourceGridPolar(real_space=[0,0,0],
-                 direction_space=direction_space,
+        return SourceGridPolar(real_space_width=[0,0,0],
+                 direction_space_width=direction_space_width,
                  real_space_points=[1,1,1],
                  direction_space_points=direction_space_points,
                  real_space_center=real_space_center,
@@ -69,21 +74,21 @@ class SourceGridPolar(S4LightSourceBase):
 
     @classmethod
     def initialize_collimated_source(cls,
-                real_space=[1e-6,0.0,1e-6],
+                real_space_width=[1e-6,0.0,1e-6],
                 real_space_points=[100,36],
                 real_space_center=[0.0,0.0,0.0],
                 direction_space_center=[0.0,0.0],
                                 ):
         """
 
-        :param real_space: Default: [1e-6,0,1e-6]
+        :param real_space_width: Default: [1e-6,0,1e-6]
         :param real_space_points: Default: [100,36]
         :param real_space_center: Default: [0.0,0.0,0.0]
         :param direction_space_center: Default: [0.0,0.0]
         :return:
         """
-        return SourceGridPolar(real_space=real_space,
-                 direction_space=[0.0,0.0],
+        return SourceGridPolar(real_space_width=real_space_width,
+                 direction_space_width=[0.0,0.0],
                  real_space_points=real_space_points,
                  direction_space_points=[1,1],
                  real_space_center=real_space_center,
@@ -136,8 +141,8 @@ class SourceGridPolar(S4LightSourceBase):
         for radius_ratio in radial_ratio:
             for angle in angular:
                 i += 1
-                x[i] = self._real_space[0]/2 * radius_ratio * numpy.cos(angle)
-                z[i] = self._real_space[2]/2 * radius_ratio * numpy.sin(angle)
+                x[i] = self._real_space_width[0]/2 * radius_ratio * numpy.cos(angle)
+                z[i] = self._real_space_width[2]/2 * radius_ratio * numpy.sin(angle)
         y = numpy.zeros_like(x)
         return x,y,z
 
@@ -161,8 +166,8 @@ class SourceGridPolar(S4LightSourceBase):
         for radius_ratio in radial_ratio:
             for angle in angular:
                 i += 1
-                vx[i] = self._direction_space[0]/2 * radius_ratio * numpy.cos(angle)
-                vz[i] = self._direction_space[0]/2 * radius_ratio * numpy.sin(angle)
+                vx[i] = self._direction_space_width[0]/2 * radius_ratio * numpy.cos(angle)
+                vz[i] = self._direction_space_width[0]/2 * radius_ratio * numpy.sin(angle)
 
         return vx,vz
 
@@ -176,7 +181,6 @@ class SourceGridPolar(S4LightSourceBase):
         """
         X, Y, Z = self.get_arrays_real_space()
         VX, VZ = self.get_arrays_direction_space()
-        print(">>>", X.shape, Y.shape, Z.shape, VX.shape, VZ.shape)
 
         npoint = self.get_number_of_points()
         V1x = numpy.zeros(npoint)
@@ -231,8 +235,8 @@ class SourceGridPolar(S4LightSourceBase):
         txt += "Gridding in direction space: %d, %d \n"%(self._direction_space_points[0],
                                                                 self._direction_space_points[1])
         txt += "\n"
-        txt += "real_space widths"+repr(self._real_space) + "\n"
-        txt += "direction_space widths "+repr(self._direction_space) + "\n"
+        txt += "real_space widths"+repr(self._real_space_width) + "\n"
+        txt += "direction_space widths "+repr(self._direction_space_width) + "\n"
         txt += "real_space_points "+repr(self._real_space_points) + "\n"
         txt += "direction_space_points "+repr(self._direction_space_points) + "\n"
         txt += "real_space_center "+repr(self._real_space_center) + "\n"
@@ -243,12 +247,13 @@ class SourceGridPolar(S4LightSourceBase):
         return txt
 
 
-    def get_beam(self,wavelength=1e-10):
-        """
-        Returns a Beam
-        :param wavelength: the photon wavelength in m
-        :return:
-        """
+    def get_beam(self):
+
+        # obtain polarization
+        DENOM = numpy.sqrt(1.0 - 2.0 * self._polarization_degree + 2.0 * self._polarization_degree ** 2)
+        AX = self._polarization_degree / DENOM
+        AZ = (1.0 - self._polarization_degree) / DENOM
+
 
         rays = numpy.zeros((self.get_number_of_points(),18))
         volume = self.get_volume()
@@ -258,22 +263,44 @@ class SourceGridPolar(S4LightSourceBase):
         rays[:, 3] = volume[3,:]
         rays[:, 4] = volume[4,:]
         rays[:, 5] = volume[5,:]
-        rays[:,6] = 1.0 # Es
+        rays[:,6] = AX # Es
         rays[:,9] = 1   # flag
-        rays[:,10] = 2 * numpy.pi / (wavelength * 1e2) # wavenumber
+        rays[:,10] = 2 * numpy.pi / (self._wavelength * 1e2) # wavenumber in cm**-1
         rays[:,11] = numpy.arange(self.get_number_of_points(),dtype=float) # index
+        rays[:, 14] = self._polarization_phase_deg # Phase p
+        rays[:, 17] = AZ # Ep
 
-        return Beam.initialize_from_array(rays)
+        return S4Beam.initialize_from_array(rays)
 
+    def to_python_code(self):
+
+        txt = ""
+        txt += "\n#\n#\n#"
+
+        txt += "\nfrom shadow4.sources.source_geometrical.source_grid_polar import SourceGridPolar"
+        txt += "\nlight_source = SourceGridPolar(name='%s', " % (self.get_name())
+        txt += "\n   real_space_width = [%f, %f, %f]," % (tuple(self._real_space_width))
+        txt += "\n   real_space_center = [%f, %f, %f]," % (tuple(self._real_space_center))
+        txt += "\n   real_space_points = [%d, %d]," % (tuple(self._real_space_points))
+        txt += "\n   direction_space_width = [%f, %f]," % (tuple(self._direction_space_width))
+        txt += "\n   direction_space_center = [%f, %f]," % (tuple(self._direction_space_center))
+        txt += "\n   direction_space_points = [%d, %d]," % (tuple(self._direction_space_points))
+        txt += "\n   wavelength = %g," % self._wavelength
+        txt += "\n   polarization_degree = %g," % self._polarization_degree
+        txt += "\n   polarization_phase_deg = %g)" % self._polarization_phase_deg
+
+        txt += "\nbeam = light_source.get_beam()"
+
+        return txt
 
 if __name__ == "__main__":
     from srxraylib.plot.gol import plot_scatter, set_qt
     from shadow4tests.compatibility.beam3 import Beam3
     set_qt()
 
-    if True:
+    if False:
         a = SourceGridPolar.initialize_point_source(
-                    direction_space        = [2e-3,2e-3],
+                    direction_space_width  = [2e-3,2e-3],
                     direction_space_points = [20,  5],
                     direction_space_center = [0.0, 0.0] )
         print(a.info())
@@ -290,9 +317,9 @@ if __name__ == "__main__":
 
     #
     #
-    if True:
+    if False:
         a = SourceGridPolar.initialize_collimated_source(
-            real_space=[2e-6, 0.0, 1e-6],
+            real_space_width=[2e-6, 0.0, 1e-6],
             real_space_points=[10, 4],
             real_space_center=[0.0, 0.0, 0.0]
         )
@@ -311,12 +338,12 @@ if __name__ == "__main__":
         # #
         #
 
-    if True:
+    if False:
         a = SourceGridPolar(
-            real_space=[1e-6, 0.0, 1e-6],
+            real_space_width=[1e-6, 0.0, 1e-6],
             real_space_points=[2, 4],
             real_space_center=[0.0, 0.0, 0.0],
-            direction_space=[2e-6, 2e-6],
+            direction_space_width=[2e-6, 2e-6],
             direction_space_points=[5, 5],
             direction_space_center=[0.0, 0.0])
 
@@ -328,7 +355,10 @@ if __name__ == "__main__":
                      title="Real space")
         plot_scatter(beam.get_column(4)*1e6, beam.get_column(6)*1e6, xrange=[-1,1], yrange=[-1,1],
                      title="Directions space")
-
+        plot_scatter(beam.get_column(1)*1e6, beam.get_column(4)*1e6, xrange=[-1,1], yrange=[-1,1],
+                     title="Phase space X")
+        plot_scatter(beam.get_column(3)*1e6, beam.get_column(6)*1e6, xrange=[-1,1], yrange=[-1,1],
+                     title="Phase space Z")
         beam_shadow3 = Beam3.initialize_from_shadow4_beam( beam )
         # beam_shadow3.write("begin.dat")
 
@@ -336,4 +366,34 @@ if __name__ == "__main__":
         Shadow.ShadowTools.plotxy(beam_shadow3, 1, 3)
         Shadow.ShadowTools.plotxy(beam_shadow3, 4, 6)
         # #
-        #
\ No newline at end of file
+        #
+
+    if False:
+        a = SourceGridPolar(
+            real_space_width=[2e-3, 0.0, 2e-3],
+            real_space_points=[2, 8],
+            real_space_center=[0.0, 0.0, 0.0],
+            direction_space_width=[20e-3, 20e-3],
+            direction_space_points=[3, 359],
+            direction_space_center=[0.0, 0.0])
+
+        print(a.info())
+
+
+        beam = a.get_beam()
+        plot_scatter(beam.get_column(1)*1e6, beam.get_column(3)*1e6, xrange=[-1.1e3,1.1e3], yrange=[-1.1e3,1.1e3], title="Real space")
+        plot_scatter(beam.get_column(4)*1e6, beam.get_column(6)*1e6, xrange=[-11e3,11e3], yrange=[-11e3,11e3], title="Directions space")
+        plot_scatter(beam.get_column(1)*1e6, beam.get_column(4)*1e6, xrange=[-1.1e3,1.1e3], yrange=[-11e3,11e3], title="Phase space X")
+        plot_scatter(beam.get_column(3)*1e6, beam.get_column(6)*1e6, xrange=[-1.1e3,1.1e3], yrange=[-11e3,11e3], title="Phase space Z")
+        beam_shadow3 = Beam3.initialize_from_shadow4_beam( beam )
+
+    if True:
+        a = SourceGridPolar(
+            real_space_width=[2e-3, 0.0, 2e-3],
+            real_space_points=[2, 8],
+            real_space_center=[0.0, 0.0, 0.0],
+            direction_space_width=[20e-3, 20e-3],
+            direction_space_points=[3, 359],
+            direction_space_center=[0.0, 0.0])
+
+        print(a.to_python_code())
\ No newline at end of file
